#include <sps2lib.h>
#include <sps2tags.h>
#include <sps2util.h>
#include <sps2regstructs.h>
#include <assert.h>
#include <memory.h>
#include <signal.h>
#include "PS2Defines.h"
#include "ps2maths.h"
#include "ps2matrix4x4.h"
#include "pad.h"
#include "sps2wrap.h"
#include "dma.h"
#include "texture.h"
#include "font.h"

bool g_bLoop = true;

void sig_handle(int sig)
{
	g_bLoop = false;
}

int main()
{
	// Initialise control pad 0
	if(!pad_init(PAD_0, PAD_INIT_LOCK | PAD_INIT_ANALOGUE | PAD_INIT_PRESSURE))
	{
		printf("Failed to initialise control pad\n");
		pad_cleanup(PAD_0);
		exit(0);
	}	
	
	// Open VU0. It is a good idea to do this as it makes sure that
	// no other programs are using VU0 at the same time
	int iVPU0; iVPU0=open("/dev/ps2vpu0", O_RDWR);

	SPS2Manager.Initialise(4096);	// 4096 * 4K Pages = 16MB Total

	VIFStaticDMA.Initialise(1536);	// 1536 * 4K Pages = 6MB Static DMA
	VIFDynamicDMA.Initialise(256);	// 256 * 4K Pages * 2 Buffers =
									// 2MB Dynamic DMA

	// Register our signal function for every possible signal (i.e. ctrl + c)
	for(int sig=0; sig<128; sig++)
		signal(sig, sig_handle);

	// Set up the DMA packet to clear the screen. We want to clear to blue.
	SPS2Manager.InitScreenClear(0x00, 0x00, 0x00);
	
	// Load the font bitmap and data
	CFont Font;
	CTexture FontTex;
	if(!Font.Load("font.dat", true))
	{
		g_bLoop = false;
		printf("Can't load font.dat\n");
	}
	if(!FontTex.LoadBitmap("font.bmp", false, true))
	{
		printf("Can't load font.bmp\n");
		g_bLoop = false;
	}
			FontTex.Upload(TEXBUF480);
	int Frame = 0;
	
	while(g_bLoop)
	{
		VIFDynamicDMA.Fire();
		
		pad_update(PAD_0);

		SPS2Manager.BeginScene();
		

		FontTex.Select();
		
		
		// VU0 VECTOR ADD EXAMPLE
		float fVecAns[4] ALIGN_QW;
		float fVecX[4]   ALIGN_QW = {0.14f, 0.53f, 0.22f, 1.05f};
		float fVecY[4]   ALIGN_QW = {0.14f, 0.53f, 0.22f, 1.05f};

		asm __volatile__(" 
			
			lqc2 vf1, 0(%1)				# vf1 = fVecX
			lqc2 vf2, 0(%2)				# vf2 = fVecY
			vadd.xyzw vf3, vf1, vf2		# vf3 = vf1 + vf2
			sqc2 vf3, 0(%0)				# fVecAns = vf3
			
		": : "r" (fVecAns) , "r" (fVecX), "r" (fVecY)); 

		Font.printfL(-200,-200, 0x10, 0x80, 0x20, 0x80,
		"| %.3f |   | %.3f |   | %.3f |\n" \
		"| %.3f | + | %.3f | = | %.3f |\n" \
		"| %.3f |   | %.3f |   | %.3f |\n" \
		"| %.3f |   | %.3f |   | %.3f |\n",
		fVecX[0], fVecY[0], fVecAns[0],
		fVecX[1], fVecY[1], fVecAns[1],
		fVecX[2], fVecY[2], fVecAns[2],
		fVecX[3], fVecY[3], fVecAns[3]);

		// VU0 VECTOR CROSS PRODUCT EXAMPLE
		float fVecX2[4] ALIGN_QW = {1.0f, 0.0f, 0.0f, 0.0f};
		float fVecY2[4] ALIGN_QW = {0.0f, 1.0f, 0.0f, 0.0f};

		asm __volatile__(" 
			
			lqc2 vf1, 0(%1)						# vf1 = fVecX2
			lqc2 vf2, 0(%2)						# vf2 = fVecY2
			vopmula.xyz ACCxyz, vf1xyz, vf2xyz	# Cross product mul stage
			vopmsub.xyz vf3xyz, vf2xyz, vf1xyz	# vf3 = vf1 X vf2
			sqc2 vf3, 0(%0)						# fVecAns = vf3
			
		": : "r" (fVecAns) , "r" (fVecX2), "r" (fVecY2)); 

		Font.printfL(-200, 0, 0x10, 0x80, 0x20, 0x80,
		"| %.3f |   | %.3f |   | %.3f |\n" \
		"| %.3f | X | %.3f | = | %.3f |\n" \
		"| %.3f |   | %.3f |   | %.3f |\n",
		fVecX2[0], fVecY2[0], fVecAns[0],
		fVecX2[1], fVecY2[1], fVecAns[1],
		fVecX2[2], fVecY2[2], fVecAns[2]);

		
		
		Font.printfR(  300, -235, 127, 127, 127, 127, "Frame %d", Frame++);
		
		
		SPS2Manager.EndScene();
		
		// Check for exit condition
		if((pad[0].buttons & PAD_START)&&(pad[0].buttons & PAD_SELECT)) g_bLoop = false;
	}

	pad_cleanup(PAD_0);
	
	return 0;
}

